const { Builder, By, until } = require('selenium-webdriver'),
  chrome = require('selenium-webdriver/chrome.js'),
  chromeOptions = new chrome.Options(),
  { readFile, readdir } = require('node:fs/promises'),
  { HttpResponse } = require('selenium-webdriver/devtools/networkinterceptor');

chromeOptions.addArguments('--headless', '--disable-gpu', '--no-sandbox');

const buildPath = 'build/assets/images';
const defaultScenario = 'scenario-side_show';
const fixtureFilePath = './test/integration/fixtures/scenario-test.svg';

const selected = expect.stringContaining('selected');
const notSelected = expect.not.stringContaining('selected');

let driver, httpResponse;

beforeAll(async () => {
  const filenames = await readdir(buildPath);
  const scenario = filenames.find(filename => filename.includes(defaultScenario));

  httpResponse = new HttpResponse(url(`/assets/images/${scenario}`));
  httpResponse.body = await readFile(fixtureFilePath, 'utf8');
  httpResponse.addHeaders('Content-Type', 'image/svg+xml');
});

beforeEach(async () => {
  driver = new Builder().forBrowser('chrome').setChromeOptions(chromeOptions).build();
  const connection = await driver.createCDPConnection('page')
  await driver.onIntercept(connection, httpResponse, async () => {});
  await driver.get(url('/'));
  const mapPlaceholderEl = await driver.findElement(By.css('.map-placeholder'));
  await driver.wait(until.elementIsNotVisible(mapPlaceholderEl), 1000);
});

it('selects and deselects a trooper by clicking on its counter', async () => {
  const record = await driver.findElement(By.css('.soldier-record'));
  const objectEl = await driver.findElement(By.css('object'));
  const selector = '.counter[data-allegiance="attacker"][data-number="1"]';

  expect(await record.getAttribute('class')).toEqual(notSelected);
  await driver.switchTo().frame(objectEl);
  const svg = await driver.findElement(By.css('svg'));
  const counter = await driver.findElement(By.css(selector), svg);
  expect(await counter.getAttribute('class')).toEqual(notSelected);

  await counter.click();

  expect(await counter.getAttribute('class')).toEqual(selected);
  await driver.switchTo().defaultContent();
  expect(await record.getAttribute('class')).toEqual(selected);

  await driver.switchTo().frame(objectEl);
  await counter.click();

  expect(await counter.getAttribute('class')).toEqual(notSelected);
  await driver.switchTo().defaultContent();
  expect(await record.getAttribute('class')).toEqual(notSelected);
});

it('selects and deselects trooper by clicking on its record', async () => {
  const record = await driver.findElement(By.css('.soldier-record'));
  const objectEl = await driver.findElement(By.css('object'));
  const selector = '.counter[data-allegiance="attacker"][data-number="1"]';

  expect(await record.getAttribute('class')).toEqual(notSelected);
  await driver.switchTo().frame(objectEl);
  const svg = await driver.findElement(By.css('svg'));
  const counter = await driver.findElement(By.css(selector), svg);
  expect(await counter.getAttribute('class')).toEqual(notSelected);

  await driver.switchTo().defaultContent();
  await record.click();

  expect(await record.getAttribute('class')).toEqual(selected);
  await driver.switchTo().frame(objectEl);
  expect(await counter.getAttribute('class')).toEqual(selected);

  await driver.switchTo().defaultContent();
  await record.click();

  expect(await record.getAttribute('class')).toEqual(notSelected);
  await driver.switchTo().frame(objectEl);
  expect(await counter.getAttribute('class')).toEqual(notSelected);
});

it('selects a trooper by clicking on its counter and deselects it by clicking on its record', async () => {
  const record = await driver.findElement(By.css('.soldier-record'));
  const objectEl = await driver.findElement(By.css('object'));
  const selector = '.counter[data-allegiance="attacker"][data-number="1"]';

  expect(await record.getAttribute('class')).toEqual(notSelected);
  await driver.switchTo().frame(objectEl);
  const svg = await driver.findElement(By.css('svg'));
  const counter = await driver.findElement(By.css(selector), svg);
  expect(await counter.getAttribute('class')).toEqual(notSelected);

  await counter.click();

  expect(await counter.getAttribute('class')).toEqual(selected);
  await driver.switchTo().defaultContent();
  expect(await record.getAttribute('class')).toEqual(selected);

  await record.click();

  expect(await record.getAttribute('class')).toEqual(notSelected);
  await driver.switchTo().frame(objectEl);
  expect(await counter.getAttribute('class')).toEqual(notSelected);
});

it('selects a trooper by clicking on its record and deselects it by clicking on its counter', async () => {
  const record = await driver.findElement(By.css('.soldier-record'));
  const objectEl = await driver.findElement(By.css('object'));
  const selector = '.counter[data-allegiance="attacker"][data-number="1"]';

  expect(await record.getAttribute('class')).toEqual(notSelected);
  await driver.switchTo().frame(objectEl);
  const svg = await driver.findElement(By.css('svg'));
  const counter = await driver.findElement(By.css(selector), svg);
  expect(await counter.getAttribute('class')).toEqual(notSelected);

  await driver.switchTo().defaultContent();
  await record.click();

  expect(await record.getAttribute('class')).toEqual(selected);
  await driver.switchTo().frame(objectEl);
  expect(await counter.getAttribute('class')).toEqual(selected);

  await counter.click();

  expect(await counter.getAttribute('class')).toEqual(notSelected);
  await driver.switchTo().defaultContent();
  expect(await record.getAttribute('class')).toEqual(notSelected);
});

afterEach(async () => {
  await driver.quit();
});
